from plateo.parsers import (
    plate_from_platemap_spreadsheet,
    plate_from_list_spreadsheet,
    plate_from_nanodrop_xml_file,
    plate_from_aati_fragment_analyzer_peaktable,
    plate_from_aati_fragment_analyzer_zip,
    plate_from_dataframe)


def test_plate_from_platemap_spreadsheet():
    plate_from_platemap_spreadsheet
    pass

def test_plate_from_list_spreadsheet():
    plate_from_list_spreadsheet
    pass

def test_plate_from_nanodrop_xml_file():
    plate_from_nanodrop_xml_file
    pass

def test_plate_from_dataframe():
    plate_from_dataframe
    pass


def test_plate_from_aati_fragment_analyzer_peaktable():
    plate_from_aati_fragment_analyzer_peaktable
    pass


def test_plate_from_aati_fragment_analyzer_zip():
    plate_from_aati_fragment_analyzer_zip
    pass
