from plateo.parsers import (picklist_from_labcyte_echo_logfile,
                            picklist_from_tecan_evo_picklist_file)

def test_picklist_from_labcyte_echo_logfile():
    picklist_from_labcyte_echo_logfile
    pass

def test_picklist_from_tecan_evo_picklist_file():
    picklist_from_tecan_evo_picklist_file
    pass
