from plateo.exporters import (
    picklist_to_tecan_evo_picklist_file,
    picklist_to_labcyte_echo_picklist_file,
)


def test_picklist_to_tecan_evo_picklist_file():
    picklist_to_tecan_evo_picklist_file
    pass


def test_picklist_to_labcyte_echo_picklist_file():
    picklist_to_labcyte_echo_picklist_file
